package guru.kake.ronove

import javafx.collections.FXCollections
import javafx.scene.image.Image
import javafx.stage.Modality
import javafx.stage.StageStyle
import tornadofx.*
import kotlin.system.exitProcess

/**
 * The main window of the program
 */
class MainView : View("Ronove") {
    /** TornadoFX root */
    override val root = borderpane {

        //Main menu
        top = menubar {
            menu("File") {
                item("New project...") { action { TODO() } }
                item("New empty project") { action { (app as Ronove).project.new() } }
                separator()
                item("Open...","Shortcut+O") { action { (app as Ronove).project.open() }}
                menu("Open recent") {
                    item("Recent 1")
                    item("Recent 2")
                    item("Recent 3")
                    item("Recent 4")
                    item("Recent 5")
                }
                separator()
                item("Save","Shortcut+S") {
                    action { (app as Ronove).project.save() }
                    enableWhen {(app as Ronove).project.projectFileSet}
                }
                item("Save as...","Shortcut+Shift+S") { action { (app as Ronove).project.saveAs() }}
                item("Export...") { action { TODO() } }
                separator()
                item("Exit") {
                    action {
                        //TODO: Confirm unsaved changes
                        exitProcess(0)
                    }
                }
            }
            menu("Edit") {
                item("Add page") { action { (app as Ronove).addPage() } }
                item("Remove page") { action { (app as Ronove).removePage() } }
                separator()
                item("Add line","Shortcut+N") { action { (app as Ronove).addLine() } }
                //item("Remove line") { action { TODO() } }
                separator()
                item("Project settings...") { action { TODO() } }
            }
            menu("Machine translation") {
                item("Translate line") { action { TODO() } }
                item("Translate page") { action { TODO() } }
                item("Translate all pages") { action {
                    (app as Ronove).project.translateAllPages()
                } }
                separator()
                item("Machine translation settings...") {
                    action {
                        TranslationSettings().openModal()
                    }
                }
            }
            menu("Dictionary") {
                item("Project dictionary...")
                item("User dictionary...")
            }
            menu("Help") {
                item("Help")
                item("About")
            }
        }

        //Table view for editing data
        center {
            this += TranslationPageView(app as Ronove)
        }

        //Status bar
        bottom = hbox {
            label("MTL: Deepl | ")
            label("API quota ")
            progressbar(TranslationBackend.quotaProperty)
            label(" 48 / 500,000 | ")
            val providers = FXCollections.observableArrayList("DeepL","Google Translate")
            combobox(values = providers)
        }

        //Drawer for tabbed tools
        right = drawer(multiselect = true) {
            fixedContentSize = 300.0
            maxWidth = 100.0
            item<ImageTab> {
                maxWidth = 100.0
            }
            item<ReaderTab>()
            item<DictionaryTab>()
            item<TranslatorTab>()
        }
    }

    init {
        primaryStage.icons.add(Image("/ronove128.png"))
        SplashScreen((app as Ronove).project).openModal(stageStyle = StageStyle.UTILITY, block = true,
            modality = Modality.APPLICATION_MODAL, owner = primaryStage)
    }
}