package guru.kake.ronove

import javafx.beans.property.ReadOnlyObjectProperty
import javafx.beans.property.SimpleIntegerProperty
import javafx.scene.control.TableView
import tornadofx.App
import tornadofx.launch
import tornadofx.toProperty

/**
 * Main class for the Ronove application
 */
class Ronove: App(MainView::class, RonoveStylesheet::class) {
    /** Current project data */
    var project = Project()

    /** Property for listening to changes on selected table view cell */
    var cellSelection: ReadOnlyObjectProperty<Line> = Line().toProperty()

    /** Currently selected page */
    var currentPage = SimpleIntegerProperty(1)

    /** Amount of pages in project */
    var pageCount = SimpleIntegerProperty(project.pages.size)

    /** Reference to [TableView] in [MainView], used to call refresh on it. */
    lateinit var table: TableView<Line>

    /**
     * Change current page
     * @param page page to change to
     */
    fun changePage(page: Int) {
        currentPage.value = page
    }

    /**
     * Add new page to the end
     */
    fun addPage() {
        project.pages.add(Page())
        pageCount.set(project.pages.size)
        table.refresh()
    }

    /** Adds new line to the end of page */
    fun addLine() {
        project.pages[currentPage.value-1].lines.add(Line())
        table.refresh()
    }

    /** Removes current page */
    fun removePage() {
        project.pages.removeAt(currentPage.value-1)
        pageCount.set(project.pages.size)
        table.refresh()
    }
}

/** Program entry point */
fun main(args: Array<String>) = launch<Ronove>(args)
