import guru.kake.ronove.TranslationBackend
import guru.kake.ronove.TranslationService
import guru.kake.ronove.TranslationServiceException
import org.junit.jupiter.api.Assertions.assertEquals
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.api.Test

class MockTranslationService(private val mockKey: String = "") : TranslationService {
    init {
        if (mockKey == "wrong key") {
            throw TranslationServiceException("Mocking wrong key entry")
        }
    }
    override fun translate(text: String, sourceLang: String, destLang: String): String {
        return when (destLang) {
            "upper" -> text.uppercase()
            "lower" -> text.lowercase()
            "getkey" -> mockKey
            else -> text
        }
    }
}

class TranslationBackendTest {

    @BeforeEach fun init() {
        TranslationBackend.setService(MockTranslationService(), "Mock")
    }

    @Test fun translateSingle() {
        assertEquals(TranslationBackend("MixedCase","source","upper"),"MIXEDCASE")
        assertEquals(TranslationBackend("MixedCase","source","lower"),"mixedcase")
        TranslationBackend.destinationLanguage = "upper"
        assertEquals(TranslationBackend("MixedCase"),"MIXEDCASE")
        TranslationBackend.destinationLanguage = "lower"
        assertEquals(TranslationBackend("MixedCase"),"mixedcase")
    }

    @Test fun translateMultiple() {
        val list = listOf("One", "Two", "tHREE")
        assertEquals(TranslationBackend(list,"source","upper"), listOf("ONE","TWO","THREE"))
        assertEquals(TranslationBackend(list,"source","lower"), listOf("one","two","three"))
        TranslationBackend.destinationLanguage = "lower"
        assertEquals(TranslationBackend(list), listOf("one","two","three"))
        TranslationBackend.destinationLanguage = "upper"
        assertEquals(TranslationBackend(list), listOf("ONE","TWO","THREE"))
    }
}